
function WLAN_DL_perfanom()

more off

N=4; % Number of stations
G=4; % Number of aggregated packets

% Traffic load parameters

B(1)=0.5E6; % traffic load per station in bps
B(2)=2E6;
B(3)=1E6;
B(4)=2E6;

L(1)=G*8000; % packet size (in bits)
L(2)=G*4000;
L(3)=G*12000;
L(4)=G*2000;

lambda(1)=B(1)/L(1); % packets / second
lambda(2)=B(2)/L(2);
lambda(3)=B(3)/L(3);
lambda(4)=B(4)/L(4);

% AP and stations placement (2D)

x(1)=0; % AP x
y(1)=0; % AP y
x(2)=5; % station 1 x
y(2)=5; % station 1 y
x(3)=-5; % station 2 x
y(3)=-5; % station 2 y
x(4)=-5; % station 3 x
y(4)=5; % station 3 y

% Close
x(5)=5; % station 4 x
y(5)=-5; % station 4 y    
% Far  
x(5)=25; % station 4 x
y(5)=-25; % station 4 y    
  
  
figure  
scatter(x(1),y(1),'filled');  
hold
scatter(x,y);
xlabel('x axis');
ylabel('y axis');
axis([-30 30 -30 30]);  
grid
  
% Distances between the AP and each station

for n=1:N
  d(n)=sqrt((x(1)-x(n+1))^2+(y(1)-y(n+1))^2);  
end

% Pathloss

gamma = 3.5;
PL_1m = 15; % dB

for n=1:N
  PL(n) = PL_1m + d(n)*gamma*log10(d(n));
end

% Received Power

TP = 20; % Transmission power (in dBm)

for n=1:N
  RP(n) = TP - PL(n);
end

% Transmission Rates

for n=1:N
  R(n) = TxRate(RP(n));
end

disp('-------------------------------------------------------------------------------------------------')
disp('Distance AP-station (meters), Path-loss (dBs), Received Power (dBms), and Transmission Rate (bps)');
disp([d' PL' RP' R']);
disp('-------------------------------------------------------------------------------------------------')


% Transmission Duration

T0 = 9E-6;
SIFS = 16E-6;
DIFS = 34E-6;
L_ack = 112;
L_mac = 240;
T_PHY = 20E-6;
Backoff = 15.5*T0;

for i=1:4
  TDATA(i) = T_PHY + (L_mac + L(i))/R(i);
  TACK(i) = L_ack / R(i);
  T(i) = Backoff + TDATA(i) + SIFS + TACK(i) + DIFS + T0;
end

disp('Transmission Duration (miliseconds)');
disp(T.*1E3);

% Expected service time, normalized traffic load

EDs = 0;
lambda_total = sum(lambda);
for i=1:4
  EDs = EDs + (lambda(i)/lambda_total) * T(i);
end

disp('Expected service time (miliseconds) | Traffic Load (Erlangs)');
disp([EDs.*1E3 lambda_total*EDs]);

% Equilibrium distribution

Q=10; % Buffer size
K=Q+1; 
a= lambda_total * EDs;
for i=0:K
  pi(i+1)=(1-a)*a^i/(1-a^(K+1));
end

%disp('########################################################');
%disp('Equilibrium distribution');
%disp(pi);
%disp('########################################################');

% Performance Metrics

Pb = pi(K+1);
Pe = pi(1);
rho = lambda_total*EDs*(1-Pb);
rho_ = 1-Pe; % It must be the same

disp('Blocking Prob | Empty Prob | AP Utilization (2 ways)');
disp([Pb Pe rho rho_]);

EN=0;
for i=0:K
  EN = EN + pi(i+1)*i;  
end

ENq=0;
for i=2:K
  ENq = ENq + pi(i+1)*(i-1);  
end

ED = EN / (lambda_total*(1-Pb));
EDq = ENq / (lambda_total*(1-Pb));
EDs_ = ED-EDq; % It must be the same as the previous EDs value

disp('AP and buffer occupancy (packets) | Expected system and Waiting delay (seconds)');
disp([EN' ENq' ED' EDq']);

% System delay per station 
ED_station(1)=EDq + T(1);   
ED_station(2)=EDq + T(2);
ED_station(3)=EDq + T(3);
ED_station(4)=EDq + T(4);
ED_ = (lambda(1)/lambda_total)*ED_station(1)+(lambda(2)/lambda_total)*ED_station(2)+(lambda(3)/lambda_total)*ED_station(3)+(lambda(4)/lambda_total)*ED_station(4); % check 

for i=1:N
  S(i)=lambda(i)*L(i)*(1-Pb);   
end  
disp('-------------------------------------------------------------------------');
disp('Expected Individual System Delay (seconds)');
disp([ED_station]);
disp('Load (bps) | Throughput (bps)');
disp([B' S']);
disp('-------------------------------------------------------------------------');   
% Throughput per station

   
end


% ------------------------
% ------------------------
% ------------------------
% ------------------------


function Rate = TxRate(RP)
  
  if(RP > -40) Rate = 20E6;
  end

  if(RP <= -40 && RP > -70) Rate = 10E6;
  end

  if(RP < -70) Rate = 5E6;
  end
  
end
